# Version 0.1
#
#  GeologicTimeScale is copyright 2005 Jeffrey G. Pittman.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
#  MA 02111-1307 USA

from Products.Archetypes.public import BaseFolderSchema, Schema
from Products.Archetypes.public import StringField, IntegerField, FloatField, LinesField, BooleanField
from Products.Archetypes.public import LinesWidget, TextAreaWidget, IdWidget, IntegerWidget, DecimalWidget, StringWidget, BooleanWidget, MultiSelectionWidget
from Products.Archetypes.public import BaseFolder, registerType
from Products.GeologicTimeScale.config import PROJECTNAME
from Products.CMFCore import CMFCorePermissions

import sys,string,cStringIO

from config import *

from AccessControl import ClassSecurityInfo, ModuleSecurityInfo

from reportlab.pdfgen.canvas import Canvas

try :
    from Shared.reportlab.platypus.flowables import Image
    from Shared.reportlab.platypus.paragraph import Paragraph
    from Shared.reportlab.platypus.doctemplate import *
    from Shared.reportlab.lib.units import inch
    from Shared.reportlab.lib.units import cm
    from Shared.reportlab.lib import styles
    from Shared.PIL import Image as PIL_Image
    from reportlab.lib import colors as rlcolors
    from reportlab.graphics.shapes import *
    from reportlab.graphics.charts.textlabels import Label
    from reportlab.platypus import BaseDocTemplate, PageTemplate, Frame, Paragraph
    from reportlab.lib.styles import ParagraphStyle
    from reportlab.lib.pagesizes import A4
except ImportError :
    from reportlab.platypus import BaseDocTemplate, PageTemplate, Frame, Paragraph
    from reportlab.lib.styles import ParagraphStyle
    from reportlab.lib.pagesizes import A4
    from reportlab.platypus.flowables import Image
    from reportlab.platypus.paragraph import Paragraph
    from reportlab.platypus.doctemplate import *
    from reportlab.lib.units import inch
    from reportlab.lib.units import cm
    from reportlab.lib import styles
    from PIL import Image as PIL_Image
    from reportlab.lib import colors as rlcolors
    from reportlab.graphics.shapes import *
    from reportlab.graphics.charts.textlabels import Label

from Products.GeologicTimeScale.drawing import GeologicTimeScaleDrawing
from Products.GeologicTimeScale.sample_data import sample_point_events
from Products.GeologicTimeScale.sample_data import sample_range_events

drawing_base_schema = BaseFolderSchema +  Schema((
     StringField('id',
                 required=0, ## Still actually required, but
                             ## the widget will supply the missing value
                             ## on non-submits
                 mode="rw",
                 accessor="getId",
                 mutator="setId",
                 default=None,
                 widget=IdWidget(label="Short Name",
                                 description="Should not contain spaces, underscores or mixed case. "\
                                 "Short Name is part of the item's web address.",
                                 visible={'view' : 'visible'}),
                 ),
    StringField('title',
                required=1,
                searchable=1,
                default='',
                accessor='Title',
                widget=StringWidget(label="Title",
                                    description="This name shows in the navigation menu."),
                ),
    StringField('description',
                isMetadata=1,
                accessor='Description',
                widget=TextAreaWidget(label='Description', 
                                      description='This description is optional, but not a bad idea for search text purposes.'),
                ),
    StringField('render_types',
                vocabulary=RENDER_TYPES,
                mutator="setRenderTypes",
                accessor="RenderTypes",
                widget=MultiSelectionWidget(label='Render Types',
                                            description='Select render types to generate.',
                                            format='checkbox'),
                ),
    IntegerField('width',
                 required=1,
                 default='1000',
                 validators=("isInt"),
                 accessor='Width',
                 widget=IntegerWidget(label="Width",
                                      description="Width of Image, in pixels."),
                 ),
    IntegerField('height',
                 required=1,
                 default='4700',
                 validators=("isInt"),
                 accessor='Height',
                 widget=IntegerWidget(label="Height",
                                      description="Height of Image, in pixels."),
                 ),
    IntegerField('x_offset',
                 required=1,
                 default='25',
                 validators=("isInt"),
                 accessor='XOffset',
                 widget=IntegerWidget(label="X Offset",
                                      description="Spacing from left margin, in pixels."),
                 ),
    IntegerField('y_offset',
                 required=1,
                 default='35',
                 validators=("isInt"),
                 accessor='YOffset',
                 widget=IntegerWidget(label="Y Offset",
                                      description="Spacing from top margin, in pixels."),
                 ),
   BooleanField('scale_to_fit',
                 searchable = 0,
                 accessor='isScaleToFit',
                 widget=BooleanWidget(label='Scale to Fit',
                                      description='If yes, time scale will be scaled to fit in width x height size; if no, time scale will be cropped on right and bottom, if necessary.'),
                 ),
                ))

geologic_time_scale_schema = Schema((
    FloatField('end_time',
                 required=1,
                 default='0.0',
                 accessor='EndTime',
                 validators=("isDecimal"),
                 widget=DecimalWidget(label="End Time",
                                     description="End (Most Recent, or Top) Time for Time Scale, in millions of years."),
                 ),
    FloatField('start_time',
                 required=1,
                 default='4567.2',
                 accessor='StartTime',
                 validators=("isDecimal"),
                 widget=DecimalWidget(label="Start Time",
                                     description="Start (Earliest, or Bottom) Time for Time Scale, in millions of years."),
                 ),
   BooleanField('simple_version',
                 searchable = 0,
                 default = 1,
                 accessor='isSimpleVersion',
                 widget=BooleanWidget(label='Create Simple Version?',
                                      description='If yes, time scale will not include series within Paleozoic and Mesozoic.'),
                 ),
    LinesField('point_events_field',
               required=0,
               default='\n'.join(sample_point_events),
               accessor="PointEvents",
               validators=('validate_point_events',),
               widget=LinesWidget(label='Point Events',
                                  description='Point Events (for a point in time), one per line, with these fields: label placement (t,b,l,r,c for top, bottom, left, right, or center), x position (in pixels), label angle (0 for horizontal, 90 for vertical), symbol type (only circle, for now), width of symbol(in pixels), height of symbol (in pixels), time (millions of years ago), text for label, red (0-255), green (0-255), blue (0-255)',),
               ),
    LinesField('range_events_field',
               required=0,
               default='\n'.join(sample_range_events),
               accessor="RangeEvents",
               validators=('validate_range_events',),
               widget=LinesWidget(label='Range Events',
                                  description='Range Events (for a span of time), one per line, with these fields: label placement (t,b,l,r,c for top, bottom, left, right, or center), x position (in pixels), label angle (0 for horizontal, 90 for vertical), width of bar (in pixels), start time (millions of years ago), end time (millions of years ago), text for label, red (0-255), green (0-255), blue (0-255)',),
               ),
        ))

class GeologicTimeScale(BaseFolder):
    """
    A folder object to hold PNG, SVG, PDF and other images
    generated for a drawing type.
    """
    allowed_content_types = ('Document', 'File',)

    filter_content_types = 1

    global_allow = 1

    schema = drawing_base_schema + geologic_time_scale_schema

    content_icon = 'timescaleicon.gif'

    security = ClassSecurityInfo()

    actions = ({'id': 'view',
                'name': 'View',
                'action': 'string:${object_url}/GeologicTimeScale_view',
                'permissions': (CMFCorePermissions.View,)},
               {'id': 'references',
                'name': 'References',
                'action': 'string:${object_url}/reference_edit',
                'permissions': (CMFCorePermissions.ModifyPortalContent,),
                'visible':0},
               {'id': 'metadata',
                'name': 'Properties',
                'action': 'string:${object_url}/base_metadata',
                'permissions': (CMFCorePermissions.ModifyPortalContent,),
                'visible':0})

    def gatherInput(self):
        render_types = self.RenderTypes()
        height = self.Height()
        width = self.Width()
        x_offset = self.XOffset()
        y_offset = self.YOffset()
        end_time = self.EndTime()
        start_time = self.StartTime()
        scale_to_fit = self.isScaleToFit()
        simple_version = self.isSimpleVersion()
        point_events_input = self.PointEvents()
        range_events_input = self.RangeEvents()

        return (render_types, 
                height, width, 
                x_offset, y_offset, 
                end_time, start_time, 
                scale_to_fit, 
                simple_version, 
                point_events_input, 
                range_events_input)
            
    # processForm is called for creation
    def processForm(self, data=1, metadata=0, REQUEST=None, values=None):
        ret = BaseFolder.processForm(self, data, metadata, REQUEST, values)

        self.createDrawing(self.gatherInput())

        return ret

    # update is called for edit
    def update(self, **kwargs):
        ret = BaseFolder.update(self, **kwargs)

        self.createDrawing(self.gatherInput())

        return ret

    def createDrawing(self, input_items):
        render_types_with_messages, height, width, x_offset, y_offset, end_time, start_time, scale_to_fit, simple_version, point_events_input, range_events_input = input_items

        # trim off the parenthetic messages in the render_types
        render_types = []
        for render_type in render_types_with_messages:
            if len(render_type) > 0: # extra one returned
                render_types.append(render_type.split()[0])

        time_scale = GeologicTimeScaleDrawing(render_types, 
                                              height, width, 
                                              x_offset, y_offset, 
                                              end_time, start_time, 
                                              scale_to_fit, 
                                              simple_version, 
                                              point_events_input, 
                                              range_events_input)
    
        drawing = time_scale.getDrawing()

        # save the drawing in the selected formats
        if 'PNG' in render_types:
            existing_file = getattr(self, 'drawing.png', None)
            if existing_file:
                 self.manage_delObjects('drawing.png')

            from reportlab.graphics import renderPM
            render_result = renderPM.drawToString(drawing, fmt='PNG')
            self.invokeFactory('File', id='drawing.png')
            png_file = self['drawing.png']
            png_file.update_data(data=render_result)
    
        if 'JPG' in render_types:
            existing_file = getattr(self, 'drawing.jpg', None)
            if existing_file:
                 self.manage_delObjects('drawing.jpg')

            from reportlab.graphics import renderPM
            render_result = renderPM.drawToString(drawing, fmt='JPG')
            self.invokeFactory('File', id='drawing.jpg')
            jpg_file = self['drawing.jpg']
            jpg_file.update_data(data=render_result)

        if 'GIF' in render_types:
            existing_file = getattr(self, 'drawing.gif', None)
            if existing_file:
                 self.manage_delObjects('drawing.gif')

            from reportlab.graphics import renderPM
            render_result = renderPM.drawToString(drawing, fmt='GIF')
            self.invokeFactory('File', id='drawing.gif')
            gif_file = self['drawing.gif']
            gif_file.update_data(data=render_result)
    
        if 'TIFF' in render_types:
            existing_file = getattr(self, 'drawing.tiff', None)
            if existing_file:
                 self.manage_delObjects('drawing.tiff')

            from reportlab.graphics import renderPM
            render_result = renderPM.drawToString(drawing, fmt='TIFF')
            self.invokeFactory('File', id='drawing.tiff')
            tiff_file = self['drawing.tiff']
            tiff_file.update_data(data=render_result)
    
        if 'PCT' in render_types:
            existing_file = getattr(self, 'drawing.pct', None)
            if existing_file:
                 self.manage_delObjects('drawing.pct')

            from reportlab.graphics import renderPM
            render_result = renderPM.drawToString(drawing, fmt='PCT')
            self.invokeFactory('File', id='drawing.pct')
            pct_file = self['drawing.pct']
            pct_file.update_data(data=render_result)
    
        if 'SVG-via-ReportLab' in render_types:
            existing_file = getattr(self, 'drawing-via-ReportLab.html', None)
            if existing_file:
                 self.manage_delObjects('drawing-via-ReportLab.html')

            self.invokeFactory('File', id='drawing-via-ReportLab.html')
            svg_file = self['drawing-via-ReportLab.html']
            svg_file.update_data(data=''.join(time_scale.getHTMLForContainingSVG('drawing-via-ReportLab')))

            existing_file = getattr(self, 'drawing-via-ReportLab.svg', None)
            if existing_file:
                 self.manage_delObjects('drawing-via-ReportLab.svg')

            from reportlab.graphics import renderSVG
            render_result = renderSVG.drawToString(drawing)
            self.invokeFactory('File', id='drawing-via-ReportLab.svg')
            svg_file = self['drawing-via-ReportLab.svg']
            svg_file.update_data(data=render_result)

        if 'SVG-directly-emitted' in render_types:
            existing_file = getattr(self, 'drawing-directly-emitted.html', None)
            if existing_file:
                 self.manage_delObjects('drawing-directly-emitted.html')

            self.invokeFactory('Document', id='drawing-directly-emitted.html')
            html_file = self['drawing-directly-emitted.html']
            html_file.edit(text_format='html',text=''.join(time_scale.getHTMLForContainingSVG('drawing-directly-emitted')))
            #def edit(self, text_format, text, file='', safety_belt=''):

            existing_file = getattr(self, 'drawing-directly-emitted.svg', None)
            if existing_file:
                 self.manage_delObjects('drawing-directly-emitted.svg')

            self.invokeFactory('File', id='drawing-directly-emitted.svg')
            svg_file = self['drawing-directly-emitted.svg']
            svg_file.update_data(data=''.join(time_scale.getSVG()))
        
        #     self.invokeFactory('File', id='svgecmascript.js')
        #     ecmascript_file = self['svgecmascript.js']
        #     ecmascript_string = string.join(drawing.getEcmascript())
        #ecmascript_file.file_edit(file=dummy.File('svgecmascript.js', data=string.join(drawing.getEcmascript())))
        #     ecmascript_file.update_data(data=ecmascript_string)

        if 'PDF' in render_types:
            existing_file = getattr(self, 'drawing.pdf', None)
            if existing_file:
                 self.manage_delObjects('drawing.pdf')

            canvas = Canvas('drawing.pdf', pagesize=(drawing.width, drawing.height))
            canvas.setFont('Times-Roman', 12)
            drawing.drawOn(canvas, 0, 0)
            pdf_content = str(canvas.getpdfdata())
            self.invokeFactory('File', id='drawing.pdf')
            pdf_file = self['drawing.pdf']
            pdf_file.update_data(data=pdf_content)

        if 'PS' in render_types:
            existing_file = getattr(self, 'drawing.ps', None)
            if existing_file:
                 self.manage_delObjects('drawing.ps')

            from reportlab.graphics import renderPS
            render_result = renderPS.drawToString(drawing)
            self.invokeFactory('File', id='drawing.ps')
            ps_file = self['drawing.ps']
            ps_file.update_data(data=render_result)

        if 'HTML' in render_types:
            existing_file = getattr(self, 'drawing.html', None)
            if existing_file:
                self.manage_delObjects('drawing.html')

            self.invokeFactory('Document', id='drawing.html')
            html_file = self['drawing.html']
            html_file.edit(text_format='html',text=''.join(time_scale.getHTMLForListing()))

        if 'IMAGEMAP' in render_types:
            existing_file = getattr(self,
            'drawing-with-imagemap.html', None)
            if existing_file:
                self.manage_delObjects('drawing-with-imagemap.html')

            self.invokeFactory('Document', id='drawing-with-imagemap.html')
            html_file = self['drawing-with-imagemap.html']
            html_file.edit(text_format='html',text=''.join(time_scale.getHTMLForContainingImageMap('drawing-with-imagemap')))

registerType(GeologicTimeScale, PROJECTNAME)

