from Products.validation.interfaces import ivalidator

class RangeEventsValidator:

    __implements__ = (ivalidator,)

    def __init__(self, name):
        self.name = name

    def __call__(self, value, *args, **kwargs):
        self.lines = {'label_placement' : [],
                      'x_position'      : [],
                      'label_angle'     : [],
                      'bar_width'       : [],
                      'start_time'      : [],
                      'end_time'        : [],
                      'font_size'       : [],
                      'text'            : [],
                      'red'             : [],
                      'green'           : [],
                      'blue'            : []}
                  
        # setup and check for item count errors:

        line_errors = []
        line_count = 1
        for event_string in value:
            label_placement = ""
            label_angle = ""
            x_position = ""
            bar_width = ""
            start_time = ""
            end_time = ""
            text = ""
            font_size = ""
            red = ""
            green = ""
            blue = ""
            try:
                items = eval('[%s]' % event_string)
            except:
                return "Mixed numeric and text input somewhere on line %d" % line_count
            if len(items) != 11:
                line_errors.append("%d" % line_count)
            else:
                label_placement,label_angle,x_position,bar_width,start_time,end_time,font_size,text,red,green,blue = items
                self.lines['label_placement'].append(label_placement)
                self.lines['label_angle'].append(label_angle)
                self.lines['x_position'].append(x_position)
                self.lines['bar_width'].append(bar_width)
                self.lines['start_time'].append(start_time)
                self.lines['end_time'].append(end_time)
                self.lines['font_size'].append(font_size)
                self.lines['text'].append(text)
                self.lines['red'].append(red)
                self.lines['green'].append(green)
                self.lines['blue'].append(blue)
            line_count += 1
        if len(line_errors) > 0:
            return "Wrong number of items on lines: %s" % line_errors

        # check for basic conversion errors:

        error_message = ""
        errors = None

        errors = self.checkForLabelPlacementErrors()
        if errors:
            error_message = "Label Placement (field 1) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForLabelAngleErrors()
        if errors:
            error_message = "Label Angle (field 2) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForXPositionErrors()
        if errors:
            error_message = "X Position (field 3) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForBarWidthErrors()
        if errors:
            error_message = "Bar Width (field 4) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForStartTimeErrors()
        if errors:
            error_message = "Start Time (field 5) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForEndTimeErrors()
        if errors:
            error_message = "End Time (field 6) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForFontSizeErrors()
        if errors:
            error_message = "Text (field 7) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForTextErrors()
        if errors:
            error_message = "Text (field 8) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForRedErrors()
        if errors:
            error_message = "Red (field 9) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForGreenErrors()
        if errors:
            error_message = "Green (field 10) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForBlueErrors()
        if errors:
            error_message = "Blue (field 11) error on lines: %s" % str(errors)
            return error_message

        # check for start and end time range errors:
        error_message = ""
        errors = self.checkForStartAndEndTimeErrors()
        if errors:
            error_message = "Problem with start and/or end time on lines: %s" % str(errors)
            return error_message

        # check for color range errors:
        errors = self.checkForColorRangeErrors()
        if errors:
            error_message = "One or more color values out of range (0-255) on lines: %s" % str(errors)
            return error_message

        return 1

    def checkForLabelPlacementErrors(self):
        line_errors = []
        count = 1
        choices = ['t', 'b', 'l', 'r', 'c', 'T', 'B', 'L', 'R', 'C']
        for label_placement in self.lines['label_placement']:
            if label_placement not in choices or len(label_placement) > 1:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForLabelAngleErrors(self):
        line_errors = []
        count = 1
        for label_angle in self.lines['label_angle']:
            try:
                int(label_angle)
            except:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForXPositionErrors(self):
        line_errors = []
        count = 1
        for x_position in self.lines['x_position']:
            try:
                int(x_position)
            except:
                line_errors.append(count)
            if int(x_position) < 0:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForBarWidthErrors(self):
        line_errors = []
        count = 1
        for bar_width in self.lines['bar_width']:
            try:
                int(bar_width)
            except:
                line_errors.append(count)
            # don't let it be zero
            if int(bar_width) <= 0:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForStartTimeErrors(self):
        line_errors = []
        count = 1
        for start_time in self.lines['start_time']:
            try:
                float(start_time)
            except:
                line_errors.append(count)
            if float(start_time) < 0.0:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForEndTimeErrors(self):
        line_errors = []
        count = 1
        for end_time in self.lines['end_time']:
            try:
                float(end_time)
            except:
                line_errors.append(count)
            if float(end_time) < 0.0:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForFontSizeErrors(self):
        line_errors = []
        count = 1
        for font_size in self.lines['font_size']:
            try:
                int(font_size)
            except:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForTextErrors(self):
        # pass
        return None

    def checkForColorErrors(self, color_name):
        line_errors = []
        count = 1
        for color in self.lines[color_name]:
            try:
                int(color)
            except:
                line_errors.append(count)
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForRedErrors(self):
        return self.checkForColorErrors('red')

    def checkForGreenErrors(self):
        return self.checkForColorErrors('green')

    def checkForBlueErrors(self):
        return self.checkForColorErrors('blue')

    def checkForStartAndEndTimeErrors(self):
        line_errors = []
        for i in range(len(self.lines['start_time'])):
            start_time = float(self.lines['start_time'][i])
            end_time = float(self.lines['end_time'][i])
            if end_time > start_time:
                line_errors.append(i+1)
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForColorRangeErrors(self):
        line_errors = []
        for i in range(len(self.lines['red'])):
            red = int(self.lines['red'][i])
            green = int(self.lines['green'][i])
            blue = int(self.lines['blue'][i])

            # must be between 0 and 255
            if red < 0 or red > 255 or green < 0 or green > 255 or blue < 0 or blue > 255:
                line_errors.append(i+1)
        if len(line_errors) > 0:
            return line_errors
        return None
