from Products.GeologicTimeScale.utils import getrlcolor
from Products.GeologicTimeScale.utils import colorstr
from Products.GeologicTimeScale.utils import wordWrap

from reportlab.graphics.charts.textlabels import Label

from reportlab.graphics.shapes import Line as RLLine
from reportlab.graphics.shapes import Rect as RLRect
from reportlab.graphics.shapes import Circle as RLCircle
from reportlab.graphics.shapes import Ellipse as RLEllipse
from reportlab.graphics.shapes import PolyLine as RLPolyLine
from reportlab.graphics.charts.textlabels import Label

# The following are wrappers around the ReportLab shapes, 
# for the rl_draw methods, and also provide strarray methods
# for directly emitting SVG.
#
class Line:
    def __init__(self,start,end,width,color):
        self.start = start #xy tuple
        self.end = end     #xy tuple
        self.width = width
        self.color = color
        return

    def rl_draw(self, drawing):
        drawing.add(RLLine(self.start[0],self.start[1],self.end[0],self.end[1],getrlcolor(self.color[0],self.color[1],self.color[2]),strokeLineCap=0))

    def strarray(self):
        return ["    <line x1=\"%d\" " % self.start[0],
                "y1=\"%d\" " % self.start[1],
                "x2=\"%d\" " % self.end[0], 
                "y2=\"%d\" " % self.end[1],
                "style=\"stroke:%s;" % colorstr(self.color),
                "stroke-width:%d;\" />\n" % self.width]

class DashedLine:
    def __init__(self,start,end,width,color,dasharray):
        self.start = start #xy tuple
        self.end = end     #xy tuple
        self.width = width
        self.color = color
        self.dasharray = dasharray # comes in as string
        return

    def rl_draw(self, drawing):
        drawing.add(RLLine(self.start[0],self.start[1],self.end[0],self.end[1],getrlcolor(self.color[0],self.color[1],self.color[2]),strokeLineCap=0))

    def strarray(self):
        return ["    <line x1=\"%d\" " % self.start[0],
                "y1=\"%d\" " % self.start[1],
                "x2=\"%d\" " % self.end[0], 
                "y2=\"%d\" " % self.end[1],
                "style=\"stroke:%s;" % colorstr(self.color),
                "stroke-width:%d;" % self.width,
                "stroke-dasharray:%s;\" />\n" % self.dasharray]

class Circle:
    def __init__(self,center,radius,color):
        self.center = center #xy tuple
        self.radius = radius 
        self.color = color   #rgb tuple in range(0,256)
        return

    def rl_draw(self, drawing):
        drawing.add(RLCircle(self.center[0],self.center[1],self.radius,fillColor=getrlcolor(self.color[0],self.color[1],self.color[2])))

    def strarray(self):
        return ["    <circle cx=\"%d\" cy=\"%d\" r=\"%d\" " %\
                (self.center[0],self.center[1],self.radius),
                "style=\"fill:%s;\" />\n" % colorstr(self.color)]

class Rectangle:
    def __init__(self,origin,width,height,stroke_width,color,stroke_color):
        self.origin = origin
        self.width = width
        self.height = height
        self.stroke_width = stroke_width
        self.color = color
        self.stroke_color = stroke_color
        return

    def rl_draw(self, drawing):
        drawing.add(RLRect(self.origin[0],
                           self.origin[1],
                           self.width,
                           self.height,
                           fillColor=getrlcolor(self.color[0],self.color[1],self.color[2])))

    def strarray(self):
        return ["    <rect x=\"%d\" " % self.origin[0],
                "y=\"%d\" " % self.origin[1], 
                "width=\"%d\" " % self.width,
                "height=\"%d\" " % self.height,
                "style=\"fill:%s;stroke-width:%d;stroke:%s;\" />\n" % \
                (colorstr(self.color), self.stroke_width, colorstr(self.stroke_color))]

class Text:
    def __init__(self,origin,text,color,size=24):
        self.origin = origin
        self.text = text
        self.size = size
        self.color = color
        return

    def rl_draw(self, drawing):
        label = Label()
        label.setOrigin(self.origin[0], self.origin[1])
        label.fillColor = getrlcolor(self.color[0],self.color[1],self.color[2])
        label.fontSize = self.size
        label.setText(self.text)
        drawing.add(label)

    def strarray(self):
        return ["    <text x=\"%d\" y=\"%d\" font-size=\"%d\" fill=\"%s\">" %\
                (self.origin[0],self.origin[1],self.size,colorstr(self.color)),
                "%s" % self.text,
                "</text>\n"]
        
class TransformedText:
    def __init__(self,origin,text,color,size=24,translation_x=0,translation_y=0,rotation=0):
        self.origin = origin
        self.text = text
        self.color = color
        self.size = size
        self.rotation = rotation
        return

    def strarray(self):
        return ["    <g transform=\"translate(%d,%d)\">\n" %\
                (self.origin[0],self.origin[1]),
                "        <g transform=\"rotate(%d)\">\n" %\
                (self.rotation),
                "            <text font-size=\"%d\" fill=\"%s\">" %\
                (self.size,colorstr(self.color)),
                "%s" % self.text,
                "</text>\n",
                "        </g>\n",
                "    </g>\n"]
        
class AlignedText:
    def __init__(self,origin,text,color,size=24,alignment="start"):
        self.origin = origin
        self.text = text
        self.color = color
        self.size = size
        self.alignment = alignment # start, middle, or end
        return

    def strarray(self):
        return ["    <text x=\"%d\" y=\"%d\" font-size=\"%d\" fill=\"%s\" text-anchor=\"%s\">" %\
                (self.origin[0],self.origin[1],self.size,colorstr(self.color),self.alignment),
                "%s" % self.text,
                "</text>\n"]
        
class TransformedAndAlignedText:
    def __init__(self,origin,text,color,size=24,rotation=0,alignment="start"):
        self.origin = origin
        self.text = text
        self.color = color
        self.size = size
        self.rotation = rotation
        self.alignment = alignment # start, middle, or end
        return

    def strarray(self):
        return ["    <g transform=\"translate(%d,%d)\">\n" %\
                (self.origin[0],self.origin[1]),
                "        <g transform=\"rotate(%d)\">\n" %\
                (self.rotation),
                "            <text font-size=\"%d\" fill=\"%s\" text-anchor=\"%s\" >" %\
                (self.size,colorstr(self.color),self.alignment),
                "%s" % self.text,
                "</text>\n",
                "        </g>\n",
                "    </g>\n"]
        

